/***************************************************************************
                           elementcommander.cpp
                           -------------------------------
    copyright            : (C) 1999-2003 by Stefan Preis
                         : (C) 2004-2008 Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-10-26 21:59:44 $ 
                           $Revision: 1.12 $
***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
 this code provides a GUI to control analog turntables, shifting brigdes ...
****************************************************************************/

#include "elementcommander.h"
#include "preferences.h"
#include "element.h"

/*button icons*/
#include "pixmaps/tt_stop.xpm"
#include "pixmaps/tt_left.xpm"
#include "pixmaps/tt_left_step.xpm"
#include "pixmaps/tt_right.xpm"
#include "pixmaps/tt_right_step.xpm"



elementCommander::elementCommander(QWidget* parent, int cid)
    : QDialog(NULL), classid(cid)
{
    setObjectName("elementCommander");

    // create a button group for standard buttons ...
    buttonGB = new QGroupBox(this);
    hbox = new QHBoxLayout();
    buttonGB->setLayout(hbox);
    buttonGB->setObjectName("commandBG");
    buttonGB->setFlat(true);

    pixButton = QPixmap(tt_stop_xpm);
    buttStop = new QPushButton(pixButton, "o", this); // ... and a stop button
    hbox->addWidget(buttStop);
    buttStop->setEnabled(false);
    connect(buttStop, SIGNAL(clicked()), this, SLOT(slotStop()));

    if (classid == element::siciSbn)   // setup the remaining buttons
        setupBridge();
    else
        setupMotor();

    // fix the window's geometry
    setFixedWidth(285);
    setFixedHeight(30);
}


void elementCommander::setupBridge()
{
    pixButton = QPixmap(tt_left_step_xpm);
    buttMoveUp = new QPushButton(pixButton, "<", buttonGB);
    buttMoveUp->setCheckable(true);
    hbox->addWidget(buttMoveUp);

    pixButton = QPixmap(tt_right_step_xpm);
    buttMoveDown = new QPushButton(pixButton, ">", buttonGB);
    buttMoveDown->setCheckable(true);
    hbox->addWidget(buttMoveDown);

    setWindowTitle(tr("Shifting bridge commander"));
    connect(buttMoveUp, SIGNAL(clicked()), this, SLOT(slotMoveUp()));
    connect(buttMoveDown, SIGNAL(clicked()), this, SLOT(slotMoveDown()));

    buttMoveUp->setToolTip(tr("Move bridge upwards"));
    buttMoveDown->setToolTip(tr("Move bridge downwards"));
    buttStop->setToolTip(tr("Stop moving bridge"));
}


void elementCommander::setupMotor()
{
    pixButton = QPixmap(tt_left_xpm);
    buttRotateLeft = new QPushButton(pixButton, "<<", buttonGB);
    buttRotateLeft->setCheckable(true);
    hbox->addWidget(buttRotateLeft);

    pixButton = QPixmap(tt_right_xpm);
    buttRotateRight = new QPushButton(pixButton, ">>", buttonGB);
    buttRotateRight->setCheckable(true);
    hbox->addWidget(buttRotateRight);

    setWindowTitle(tr("DC-motor commander"));
    connect(buttRotateLeft, SIGNAL(clicked()),
            this, SLOT(slotRotateLeft()));
    connect(buttRotateRight, SIGNAL(clicked()),
            this, SLOT(slotRotateRight()));

    buttRotateLeft->setToolTip(tr("Move motor clockwise"));
    buttRotateRight->setToolTip(tr("Move motor anti-clockwise"));
    buttStop->setToolTip(tr("Stop motor"));
}


void elementCommander::slotMoveUp()
{
    buttStop->setEnabled(true);
    buttMoveUp->setEnabled(false);
    buttMoveDown->setEnabled(false);
    emit sendTtCommand(1, 0);         // select direction upwards
    emit sendTtCommand(2, 0);         // start moving bridge
}


void elementCommander::slotMoveDown()
{
    buttStop->setEnabled(true);
    buttMoveUp->setEnabled(false);
    buttMoveDown->setEnabled(false);
    emit sendTtCommand(1, 1);         // select direction downwards
    emit sendTtCommand(2, 0);         // start moving bridge
}


void elementCommander::slotRotateLeft()
{
    buttStop->setEnabled(true);
    buttRotateLeft->setEnabled(false);
    buttRotateRight->setEnabled(false);
    emit sendTtCommand(1, 0);         // select left rotating
    emit sendTtCommand(2, 0);
}


void elementCommander::slotRotateRight()
{
    buttStop->setEnabled(true);
    buttRotateLeft->setEnabled(false);
    buttRotateRight->setEnabled(false);
    emit sendTtCommand(1, 1);         // select right rotating
    emit sendTtCommand(2, 1);
}


void elementCommander::slotStop()
{
    buttStop->setEnabled(false);

    if (classid == element::siciSbn) {
        buttMoveUp->setEnabled(true);
        buttMoveDown->setEnabled(true);
        buttMoveUp->setDown(false);
        buttMoveDown->setDown(false);
        emit sendTtCommand(2, 1);     // stop moving
    }
    else if (classid == element::siciMdc) {
        buttRotateLeft->setEnabled(true);
        buttRotateRight->setEnabled(true);
        buttRotateLeft->setDown(false);
        buttRotateRight->setDown(false);
        emit sendTtCommand(1, 0);     // stop rotating with a red and a green button
        emit sendTtCommand(2, 1);
    }
}

