/*
  Copyright (c) 2014 Guido Scholz <gscholz@users.sourceforge.net>

  This file is part of spdrs60.

  spdrs60 is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2,
  or (at your option) any later version as published by the Free
  Software Foundation.

  spdrs60 is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with spdrs60.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <QAction>
#include <QMenu>
#include <QPainter>

#include "counterdialog.h"
#include "externalcounterpanel.h"


static const char GF_COUNTERVALUE[] = "countervalue";


ExternalCounterPanel::ExternalCounterPanel(QWidget* parent,
        SpdrItemClassId cid, GbsButtonState cb, const char* bt):
    ExternalButtonPanel(parent, cid, cb, bt), countervalue(0)
{
    setObjectName("ExternalCounterPanel");
}


ExternalCounterPanel::ExternalCounterPanel(QTextStream& ts, QWidget*
        parent, SpdrItemClassId cid, GbsButtonState cb, const char* bt):
    ExternalButtonPanel(ts, parent, cid, cb, bt), countervalue(0)
{
    setObjectName("ExternalCounterPanel");
    readFileTextFromStream(ts);
}


void ExternalCounterPanel::buttonTriggered()
{
    ++countervalue;
    if (countervalue == 10000)
        countervalue = 0;
    setupElementIcon();
    emit buttonClicked(ctrlButton);
}


void ExternalCounterPanel::readFileTextFromStream(QTextStream& ats)
{
    QString s, key, value;

    while (!ats.atEnd()) {
        s = ats.readLine();
        if (!s.startsWith("#")) {
            key = s.section(DS, 0, 0);
            value = s.section(DS, 1, 1).trimmed();

            /* key/value pairs are read sequence independent */
            if (key.compare(GF_INDEX) == 0) {
                  iSoldIndex = value.trimmed().toUInt();
            }
            /*read also parameter of externalbuttonpanel*/
            else if (key.compare(GF_BUTTON1FB) == 0) {
                enable1fbtrigger = value.toUInt();
                value = s.section(DS, 2, 2).trimmed();
                button1fbbus = value.toUInt();
                value = s.section(DS, 3, 3).trimmed();
                button1fbcontact = value.toUInt();
            }
            else if (key.compare(GF_COUNTERVALUE) == 0) {
                countervalue = value.toUInt();
            }
            else if (key.startsWith('%')) {
                /*end of dataset, exit while loop*/
                  return;
            }
        }
    }
}


void ExternalCounterPanel::writeFileTextToStream(QTextStream& ts)
{
    ts << GF_CLASSID << DS << classid << Qt::endl
        << GF_INDEX << DS << iSoldIndex << Qt::endl;
    /*save also parameter of externalbuttonpanel*/
    ts << GF_BUTTON1FB << DS << enable1fbtrigger << DS
        << button1fbbus << DS << button1fbcontact << Qt::endl;
    ts << GF_COUNTERVALUE << DS << countervalue << Qt::endl;
    ts << '%' << Qt::endl;
    modified = false;
}


void ExternalCounterPanel::showCounterDialog()
{
    CounterDialog* dlg = new CounterDialog(this);
    dlg->move(QCursor::pos());
    dlg->setCounterValue(countervalue);
    if (dlg->exec() == QDialog::Accepted) {
        countervalue = dlg->getCounterValue();
        setupElementIcon();
        modified = true;
    }
    delete dlg;
}

/*setup property menu and keep click position */
void ExternalCounterPanel::runPropertyMenue(const QPoint& p)
{
    QMenu* propmenu = new QMenu(this);
    propmenu->setObjectName("propertyMenu");

    QAction* triggerbuttonaction = propmenu->addAction(
            tr("Trigger &button..."));
    triggerbuttonaction->setData(1);

    QAction* setCounterAction = propmenu->addAction(
            tr("&Set counter..."));
    setCounterAction->setData(2);

    if (!propmenu->isEmpty()) {
        QAction* ac = propmenu->exec(QCursor::pos());
        if (ac != NULL) {
            switch(ac->data().toInt()) {
                case 1:
                    showFeedbackTriggerDialog(p);
                    break;
                case 2:
                    showCounterDialog();
                    break;
                case -1: //fall through
                default:
                    break;
            }
        }
    }
    delete propmenu;
}
