/***************************************************************************
                           newlayoutdialog.cpp
                           version 0.5.6 $Revision: 1.27 $
                           -------------------------------
    copyright            : (C) 1999-2003 by Stefan Preis
                           (C) 2004-2008 by Guido Scholz
    email                : guido.scholz@bayernline.de
    last modified        : $Date: 2009-10-27 20:54:53 $
****************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/***************************************************************************
   this code provides an user interface to enter the number of new columns
 ***************************************************************************/

#include <QButtonGroup>
#include <QGroupBox>
#include <QLayout>
#include <QValidator>
#include <QDialogButtonBox>

#include "newlayoutdialog.h"
#include "preferences.h"
#include "resources.h"


newLayoutDialog::newLayoutDialog(QWidget* parent)
: QDialog(parent)
{
    setWindowTitle(tr("Create new layout"));
    tabWidget = new QTabWidget(this);

    setupGeneralTab();
    setupCrcfTab();

    QDialogButtonBox* buttonBox = new QDialogButtonBox(
            QDialogButtonBox::Ok | QDialogButtonBox::Cancel);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addWidget(tabWidget);
    mainLayout->addWidget(buttonBox);
    setLayout(mainLayout);
}

void newLayoutDialog::setupGeneralTab()
{
    QWidget *w = new QWidget(this);
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);

    // layout dimensions group box
    QGroupBox* dimensionsGB = new QGroupBox(tr("Layout dimensions"), w);
    dimensionsGB->setObjectName("dimensionsGB");
    tabL->addWidget(dimensionsGB);
    QVBoxLayout* boxL = new QVBoxLayout(dimensionsGB);
    boxL->setSpacing(6);


    //line with column number
    QHBoxLayout* columnsLayout = new QHBoxLayout();
    boxL->addLayout(columnsLayout);
    QLabel* label = new QLabel(tr("&Columns:"), dimensionsGB);
    columnsLayout->addWidget(label);
    columnsLayout->addStretch();
    sbEnterCols = new QSpinBox(dimensionsGB);
    sbEnterCols->setMinimum(MIN_COLS);
    sbEnterCols->setMaximum(MAX_COLS);
    columnsLayout->addWidget(sbEnterCols);
    label->setBuddy(sbEnterCols);
    sbEnterCols->setWrapping(true);     // enables to spin "over" the limits
    sbEnterCols->setToolTip(tr(
                "Choose or enter the number of\n"
                "columns for your layout.\n"
                "Valid range is %1..%2.").arg(MIN_COLS).arg(MAX_COLS));

    //line with row number
    QHBoxLayout* rowsLayout = new QHBoxLayout();
    boxL->addLayout(rowsLayout);
    label = new QLabel(tr("&Rows:"), dimensionsGB);
    rowsLayout->addWidget(label);
    rowsLayout->addStretch();
    sbEnterRows = new QSpinBox(dimensionsGB);
    sbEnterRows->setMinimum(MIN_ROWS);
    sbEnterRows->setMaximum(MAX_ROWS);
    rowsLayout->addWidget(sbEnterRows);
    label->setBuddy(sbEnterRows);
    sbEnterRows->setWrapping(true);     // enables to spin "over" the limits
    sbEnterRows->setToolTip(tr(
                "Choose or enter the number\n"
                "of rows for your layout.\n"
                "Valid range is %1..%2.").arg(MIN_ROWS).arg(MAX_ROWS));

    // server group box
    QGroupBox *serverGB = new QGroupBox(tr("SRCP-Server"), w);
    tabL->addWidget(serverGB);
    QVBoxLayout* serverGBL = new QVBoxLayout(serverGB);
    serverGBL->setSpacing(6);

    // line with host name
    QHBoxLayout* hostL = new QHBoxLayout();
    serverGBL->addLayout(hostL);
    label = new QLabel(tr("&Hostname:"), serverGB);
    hostL->addWidget(label);
    hostL->addStretch();
    hostLE = new QLineEdit(serverGB);
    hostL->addWidget(hostLE);
    hostLE->setMaximumWidth(100);
    label->setBuddy(hostLE);
    hostLE->setToolTip(tr(
                "Enter the hostname or IP address\n"
                "of your SRCP server."));

    // line with port number
    QHBoxLayout* portL = new QHBoxLayout();
    serverGBL->addLayout(portL);
    label = new QLabel(tr("&Portnumber:"), serverGB);
    portL->addWidget(label);
    portL->addStretch();

    portLE = new QLineEdit(serverGB);
    portL->addWidget(portLE);
    portLE->setMaximumWidth(100);
    label->setBuddy(portLE);
    portLE->setMaxLength(5);
    QValidator* portValidator = new QIntValidator(1, 65535, serverGB);
    portLE->setValidator(portValidator);
    portLE->setToolTip(tr(
                "Enter the portnumber of your srcp service.\n"
                "Default value for SRCP 0.8 is 4303,\n"
                "for a SRCP 0.7 server choose 12345."));

    serverGBL->addStretch();

    // start options group box
    QGroupBox* startGB = new QGroupBox(tr("Actions on file loading"), this);
    tabL->addWidget(startGB);
    QVBoxLayout* startL = new QVBoxLayout();
    startGB->setLayout(startL);
    autologinCB = new QCheckBox(tr("Autoconnect to &server"), startGB);
    startL->addWidget(autologinCB);
    autologinCB->setToolTip(tr(
                "The SRCP server will be automatically\n"
                "connected when this file is loaded.\n"));

    autopowerCB = new QCheckBox(tr("Autostart layout &voltage"),
            startGB);
    startL->addWidget(autopowerCB);
    autopowerCB->setToolTip(tr(
                "Power of your layout will be automatically\n"
                "switched on after server connect.\n"));

    autosendallCB = new QCheckBox(tr("Send all &solenoid states after"
                " power on"), startGB);
    startL->addWidget(autosendallCB);
    autosendallCB->setToolTip(tr(
                "The configured states of all solenoids will\n"
                "be automatically send to the SRCP server\n"
                "after layout power is switched on.\n"));

    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&General"));
}


void newLayoutDialog::setupCrcfTab()
{
    QWidget *w = new QWidget(this);
    QVBoxLayout* tabL = new QVBoxLayout(w);
    tabL->setSpacing(10);

    // switch box identification group box
    QGroupBox *switchboxGB = new QGroupBox(tr("Switchbox Identification"), w);
    tabL->addWidget(switchboxGB);
    QVBoxLayout* switchboxGBL = new QVBoxLayout(switchboxGB);
    switchboxGBL->setSpacing(6);

    // line with id
    QHBoxLayout* switchboxidL = new QHBoxLayout();
    switchboxGBL->addLayout(switchboxidL);
    QLabel* label = new QLabel(tr("&Id:"), switchboxGB);
    switchboxidL->addWidget(label);
    switchboxidL->addStretch();
    switchboxidLE = new QLineEdit(switchboxGB);
    switchboxidL->addWidget(switchboxidLE);
    switchboxidLE->setMaximumWidth(100);
    switchboxidLE->setMaxLength(6);
    QValidator* idValidator = new QIntValidator(0, 999999, switchboxGB);
    switchboxidLE->setValidator(idValidator);
    label->setBuddy(switchboxidLE);
    switchboxidLE->setToolTip(tr(
                "Enter the CRCF-identification\n"
                "number of this switchbox.\n"
                "Valid range is 0..999999."));

    // line with layout name
    QHBoxLayout* switchboxnameL = new QHBoxLayout();
    switchboxGBL->addLayout(switchboxnameL);
    label = new QLabel(tr("&Name:"), switchboxGB);
    switchboxnameL->addWidget(label);
    switchboxnameL->addStretch();

    switchboxnameLE = new QLineEdit(switchboxGB);
    switchboxnameL->addWidget(switchboxnameLE);
    switchboxnameLE->setMaximumWidth(100);
    label->setBuddy(switchboxnameLE);
    switchboxnameLE->setToolTip(tr(
                "Enter the CRCF-name of this switchbox."));

    switchboxGBL->addStretch();

    // layout identification group box
    QGroupBox *layoutGB = new QGroupBox(
            tr("Layout Identification"), w);
    tabL->addWidget(layoutGB);
    QVBoxLayout* layoutGBL = new QVBoxLayout(layoutGB);
    layoutGBL->setSpacing(6);

    // line with id
    QHBoxLayout* layoutidL = new QHBoxLayout();
    layoutGBL->addLayout(layoutidL);
    label = new QLabel(tr("I&d:"), layoutGB);
    layoutidL->addWidget(label);
    layoutidL->addStretch();
    layoutidLE = new QLineEdit(layoutGB);
    layoutidL->addWidget(layoutidLE);
    layoutidLE->setMaximumWidth(100);
    layoutidLE->setMaxLength(6);
    layoutidLE->setValidator(idValidator);
    label->setBuddy(layoutidLE);
    layoutidLE->setToolTip(tr(
                "Enter the CRCF-identification\n"
                "number of this layout.\n"
                "Valid range is 0..999999."));

    // line with layout name
    QHBoxLayout* layoutnameL = new QHBoxLayout();
    layoutGBL->addLayout(layoutnameL);
    label = new QLabel(tr("N&ame:"), layoutGB);
    layoutnameL->addWidget(label);
    layoutnameL->addStretch();

    layoutnameLE = new QLineEdit(layoutGB);
    layoutnameL->addWidget(layoutnameLE);
    layoutnameLE->setMaximumWidth(100);
    label->setBuddy(layoutnameLE);
    layoutnameLE->setToolTip(tr(
                "Enter the CRCF-name of this layout."));

    layoutGBL->addStretch();

    // spacer to push group boxes to top
    tabL->addStretch();

    tabWidget->addTab(w, tr("&CRCF-Data"));
}


int newLayoutDialog::getColumns()
{
    return sbEnterCols->value();
}


int newLayoutDialog::getRows()
{
    return sbEnterRows->value();
}


unsigned int newLayoutDialog::getSwitchboxId()
{
    return switchboxidLE->text().toUInt();
}


QString newLayoutDialog::getSwitchboxName()
{
    return switchboxnameLE->text();
}


unsigned int newLayoutDialog::getLayoutId()
{
    return layoutidLE->text().toUInt();
}


QString newLayoutDialog::getLayoutName()
{
    return layoutnameLE->text();
}


void newLayoutDialog::setColumns(int cols)
{
    sbEnterCols->setValue(cols);
}


void newLayoutDialog::setRows(int rows)
{
    sbEnterRows->setValue(rows);
}


void newLayoutDialog::setSwitchboxId(unsigned int id)
{
    switchboxidLE->setText(QString::number(id));
}


void newLayoutDialog::setSwitchboxName(const QString& name)
{
    switchboxnameLE->setText(name);
}


void newLayoutDialog::setLayoutId(unsigned int id)
{
    return layoutidLE->setText(QString::number(id));
}


void newLayoutDialog::setLayoutName(const QString& name)
{
    return layoutnameLE->setText(name);
}


QString newLayoutDialog::getHost()
{
    if (hostLE->text().isEmpty())
        return "localhost"; //FIXME
    else
        return hostLE->text();
}


unsigned int newLayoutDialog::getPort()
{
    if (portLE->hasAcceptableInput())
        return portLE->text().toUInt();
    else
        return 4303;  //FIXME
}


void newLayoutDialog::setHost(const QString& host)
{
    hostLE->setText(host);
}


void newLayoutDialog::setPort(unsigned int port)
{
    portLE->setText(QString::number(port));
}


bool newLayoutDialog::getAutoLogin()
{
    return autologinCB->isChecked();
}


bool newLayoutDialog::getAutoPower()
{
    return autopowerCB->isChecked();
}


bool newLayoutDialog::getAutoSendAll()
{
    return autosendallCB->isChecked();
}


void newLayoutDialog::setAutoLogin(bool login)
{
    autologinCB->setChecked(login);
}


void newLayoutDialog::setAutoPower(bool power)
{
    autopowerCB->setChecked(power);
}


void newLayoutDialog::setAutoSendAll(bool power)
{
    autosendallCB->setChecked(power);
}

